﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using static ProjectWarmlyShip.Direction;

namespace ProjectWarmlyShip
{
    // Класс, отвечающий за прорисовку и перемещение объекта-сущности
    internal class DrawingWarmlyShip
    {
        /// <summary>
        /// Класс-сущность
        /// </summary>
        public EntityWarmlyShip? EntityWarmlyShip { get; private set; }
        /// <summary>
        ///Ширина окна
        /// </summary>
        private int _pictureWidth;
        /// <summary>
        ///Высота окна
        /// </summary>
        private int _pictureHeight;
        /// <summary>
        /// Левая координата прорисовки теплохода
        /// </summary>
        private int _startPosX;
        /// <summary>
        /// Верхняя кооридната прорисовки теплохода
        /// </summary>
        private int _startPosY;
        /// <summary>
        /// Ширина прорисовки теплохода
        /// </summary>
        private int _shipWidth = 200;
        /// <summary>
        /// Высота прорисовки теплохода
        /// </summary>
        private int _shipHeight = 30;
        /// <summary>
        /// Инициализация свойств
        /// </summary>
        /// <param name="speed">Скорость</param>
        /// <param name="weight">Вес</param>
        /// <param name="bodyColor">Цвет палубы</param>
        /// <param name="additionalColor">Дополнительный цвет</param>
        /// <param name="shipPipes">Признак наличия труб</param>
        /// <param name="width">Ширина картинки</param>
        /// <param name="height">Высота картинки</param>
        /// <returns>true - объект создан, false - проверка не пройдена, нельзя создать объект в этих размерах</returns>
        public bool Init(int speed, double weight, Color bodyColor, Color additionalColor, bool shipPipes, bool shipFuel, int width, int height)
        {
            if (width < _shipWidth || height < _shipHeight)
            {
                return false;
            }
            _pictureWidth = width;
            _pictureHeight = height;
            EntityWarmlyShip = new EntityWarmlyShip();
            EntityWarmlyShip.Init(speed, weight, bodyColor, additionalColor, shipPipes, shipFuel);
            return true;
        }
        /// <summary>
        /// Установка позиции
        /// </summary>
        /// <param name="x">Координата X</param>
        /// <param name="y">Координата Y</param>
        public void SetPosition(int x, int y)
        {
            if ((x > 0) && (x < _pictureWidth)) _startPosX = x;
            else _startPosX = 0;
            if ((y > 0) && (y < _pictureHeight)) _startPosY = y;
            else _startPosY = 0;
        }
        /// <summary>
        /// Изменение направления перемещения
        /// </summary>
        /// <param name="direction">Направление</param>
        public void MoveTransport(DirectionType direction)
        {
            if (EntityWarmlyShip == null)
            {
                return;
            }
            switch (direction)
            {
                //влево
                case DirectionType.Left:
                    if (_startPosX - EntityWarmlyShip.Step > 0)
                    {
                        _startPosX -= (int)EntityWarmlyShip.Step;
                    }
                    else
                    {
                        _startPosX = 0;
                    }
                    break;
                //вверх
                case DirectionType.Up:
                    if (_startPosY - EntityWarmlyShip.Step > 0)
                    {
                        _startPosY -= (int)EntityWarmlyShip.Step;
                    }
                    else
                    {
                        _startPosY = 0;
                    }
                    break;
                // вправо
                case DirectionType.Right:
                    if (_startPosX + _shipWidth + EntityWarmlyShip.Step < _pictureWidth)
                    {
                        _startPosX += (int)EntityWarmlyShip.Step;
                    }
                    else
                    {
                        _startPosX = _pictureWidth - _shipWidth;
                    }
                    break;
                //вниз
                case DirectionType.Down:
                    if (_startPosY + _shipHeight + EntityWarmlyShip.Step < _pictureHeight)
                    {
                        _startPosY += (int)EntityWarmlyShip.Step;
                    }
                    else
                    {
                        _startPosY = _pictureHeight - _shipHeight;
                    }
                    break;
            }
        }
        /// <summary>
        /// Прорисовка объекта
        /// </summary>
        /// <param name="g"></param>
        public void DrawTransport(Graphics g)
        {
            if (EntityWarmlyShip == null)
            {
                return;
            }
            Pen pen = new(Color.Black);
            Brush additionalBrush = new SolidBrush(EntityWarmlyShip.AdditionalColor);
            Brush bodyBrush = new SolidBrush(EntityWarmlyShip.BodyColor);
            //трубы 
            if (EntityWarmlyShip.ShipPipes)
            {
                g.FillRectangle(additionalBrush, _startPosX + 180, _startPosY + 2, 3, 12);
                g.FillRectangle(additionalBrush, _startPosX + 185, _startPosY + 4, 5, 10);
            }
            //границы корабля
            Point point1 = new Point(_startPosX, _startPosY + 12);
            Point point2 = new Point(_startPosX + 25, _startPosY + 35);
            Point point3 = new Point(_startPosX + 175, _startPosY + 35);
            Point point4 = new Point(_startPosX + 200, _startPosY + 12);

            Point[] curvePoints1 = { point1, point2, point3, point4 };
            g.FillPolygon(bodyBrush, curvePoints1);
            //граница палубы
            Brush brAqua = new SolidBrush(Color.Aquamarine);
            g.FillRectangle(brAqua, _startPosX + 50, _startPosY + 0, 125, 12);
            //топливо
            if (EntityWarmlyShip.ShipFuel)
            {
                g.FillRectangle(additionalBrush, _startPosX + 12, _startPosY + 5, 27, 7);
            }
        }
    }
}
